#!/usr/bin/perl -w

# Copyright (C) 1999 David Jao <djao@dominia.mit.edu>.
#
# This program is free software; you may redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License (http://www.gnu.org/copyleft/gpl.html)
# for more details.

package Apache::LimitByIP;

use strict;
use Apache::Constants qw(:common);
use Apache::Log ();
use IPC::Shareable ();

use vars qw(%IPUSAGE);

sub handler {
    my $r = shift;
    return DECLINED unless $r->is_initial_req;
    my $limit = $r->dir_config('LimitPerIP') || 0;
		# number of concurrent downloads allowed
    my $total_limit = $r->dir_config('TotalLimit') || 0;
    return OK if ($limit <= 0 && $total_limit <= 0); # 0 means unlimited access

# Here's an example of how to limit access to only certain file types.
# This example happens to be particularly useful for mp3 servers ^_^
#    my $content_type = $r->lookup_uri($r->uri)->content_type;
#    return OK unless $content_type =~ m:mpeg:i;

    tie %IPUSAGE, 'IPC::Shareable', 'IPLIMIT', {create => 1, mode => 0644}
    unless defined %IPUSAGE;
    
    my($ip) = ($r->connection->remote_ip);
    my $current_time = time();
    
    tied(%IPUSAGE)->shlock;
    
    my($hits, $access_time) = split ' ', $IPUSAGE{$ip};
    my($totalhits, $notime) = split ' ', $IPUSAGE{'HITS'};
	# keep track of total hits
    $access_time = $current_time;
    my $return_value = OK;
    $hits++;
    $totalhits++;
    if (($hits > $limit) && $limit) { # if download count exceeds the limit
	$r->server->log->debug("Current download count is ", $hits , ". Denying.");
	$return_value = FORBIDDEN;
	$r->log_reason("Client exceeded download limit.", $r->filename);
#   An environment variable that I personally use; you can ignore it.
#	$ENV{"badreferer"} = "toomuch";
    }
    if (($totalhits > $total_limit) && $total_limit) {
	$r->server->log->debug("Current total download count is ", $totalhits, ". Denying.");
	$return_value = FORBIDDEN;
#   An environment variable that I personally use; you can ignore it.
#	$ENV{"badreferer"} = "toomany";
    }

    $IPUSAGE{$ip} = join " ", $hits, $access_time;
    $IPUSAGE{'HITS'} = join " ", $totalhits, $notime;
    tied(%IPUSAGE)->shunlock;
    
    $r->push_handlers(PerlCleanupHandler => \&cleanup);

    return $return_value;
}
sub cleanup {
    my $r = shift;
    return DECLINED unless $r->is_initial_req;  # don't handle sub-requests

# If you pay attention to content type up there, you had better pay
# the same attention down here!
#    my $content_type = $r->lookup_uri($r->uri)->content_type;
#    return OK unless $content_type =~ m:mpeg:i;

    my $current_time = time();

  # Not strictly necessary, since the hash should have been created up above
    tie %IPUSAGE, 'IPC::Shareable', 'IPLIMIT', {create => 1, mode => 0644}
    unless defined %IPUSAGE;

    my($ip) = ($r->connection->remote_ip);
    tied(%IPUSAGE)->shlock;
    my($hits, $access_time) = split ' ', $IPUSAGE{$ip};
    my($totalhits, $notime) = split ' ', $IPUSAGE{'HITS'};
    $hits--;
    $r->server->log->debug("Download count decremented to ", $hits , ".");
    $totalhits--;
    $r->server->log->debug("Total count decremented to ", $hits , ".");
    $IPUSAGE{$ip} = join " ", $hits, $access_time;

    delete $IPUSAGE{$ip} if $hits <= 0;
    $IPUSAGE{'HITS'} = join " ", $totalhits, $notime;
      # cleaning up keys does wonders for your memory usage

    tied(%IPUSAGE)->shunlock;
}

1;
__END__

=head1 NAME

Apache::LimitByIP - Limit simultaneous connections by IP address

=head1 SYNOPSIS

 <Location /somewhere> 
 PerlAccessHandler Apache::LimitByIP
 PerlSetVar        LimitPerIP 3
 PerlSetVar        TotalLimit 30
 </Location>

=head1 DESCRIPTION

The Apache::LimitByIP module lets you enforce limits on the number of
simultaneous downloads allowed from a single IP address, and the total
number of simultaneous downloads allowed.

The LimitPerIP variable dictates the maximum number of simultaneous
connections that each IP address is allowed. Connections in excess of
this limit will result in a 403 Forbidden response. The TotalLimit
variable sets the total number of downloads that the server will allow.
Excess connection attempts will receive a 403 Forbidden response.

The job of crafting useful error messages for the client is left as an
exercise for the reader.

The default values of 0 for LimitPerIP and TotalLimit allow unlimited
connections.

Note that the limits defined by Apache::LimitPerIP apply to all IP
addresses connecting to your Apache server. Currently there is no way
to set different limits for different IP addresses.

=head1 SEE ALSO

Apache(3), IPC::Shareable(3)

=head1 CREDITS

This module borrows ideas from Apache::SpeedLimit in the mod_perl
book (http://www.modperl.com/), most notably the technique of
storing persistent state information in shared memory.

=head1 AUTHOR

David Jao <djao@dominia.mit.edu>

=head1 NOTES

This module requires IPC::Shareable and has only been tested on Linux 2.x.
